import { DynamoDB, S3 } from 'aws-sdk';
import { withCors } from '../../utils/addingCors';

const dynamoDb = new DynamoDB.DocumentClient();
const s3 = new S3();
const tableName = process.env.TABLE_NAME;
const bucketName = process.env.BUCKET_NAME;
const cloudfrontDomain = process.env.CLOUDFRONT_DOMAIN;

export const handler = async (event: any) => {
  const body = JSON.parse(event.body);

  console.log('body of Update Lambda', body);

  if (!body.name || !body.phone || !body.email) {
    return withCors(400, { error: 'Missing required fields' });
  }

  const contactId = event.pathParameters?.contactId;

  try {
    const contactResult = await dynamoDb
      .get({
        TableName: tableName!,
        Key: { id: contactId },
      })
      .promise();

    if (!contactResult.Item) {
      return withCors(404, { error: 'Contact not found' });
    }

    // Update the contact's image if a new image is provided
    let imageUrl = contactResult.Item.imageUrl;
    if (body.image) {
      const base64Image = body.image.split(',')[1];
      const imageBuffer = Buffer.from(base64Image, 'base64');
      const imageKey = `contacts/${contactId}.jpg`;

      await s3
        .putObject({
          Bucket: bucketName!,
          Key: imageKey,
          Body: imageBuffer,
          ContentType: 'image/jpeg',
          CacheControl: 'max-age=0, must-revalidate',
        })
        .promise();

      imageUrl = `https://${cloudfrontDomain}/${imageKey}?timestamp=${Date.now()}`;
    }

    // Update the contact in DynamoDB
    const updatedContact = {
      ...contactResult.Item,
      name: body.name,
      phone: body.phone,
      email: body.email,
      isFavorite: body.isFavorite || contactResult.Item.isFavorite,
      imageUrl,
    };

    await dynamoDb
      .put({
        TableName: tableName!,
        Item: updatedContact,
      })
      .promise();

    return withCors(200, updatedContact);
  } catch (error) {
    console.error(error);
    return withCors(500, { error: 'Could not update contact' });
  }
};
